/* SoccerBot - RCX Drive Control - Interface
Copyright (C) 2005 Georg Klima

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

Contact:
Peter Wild, pwild - at - cosy.sbg.ac.at
University of Salzburg, Austria
*/
package at.ac.sbg.cosy.soccerbot.rcxdrive;

/**
 * RCXDrive layer for communication with RCX
 * @author Georg Klima
 */
public interface IRCXDrive {
	/**
	 * Minimum motor speed
	 */
	public static final int MIN_SPEED = 0;
	/**
	 * Maximum motor speed
	 */
	public static final int MAX_SPEED = 255;

	/**
	 * Normal motor speed
	 */
	public static final int NORMAL_SPEED = 200;
	
	/**
	 * Sets the power
	 * @param power the power value
	 */
	public void setPower(byte power);
	
	/**
	 * Sends a command to the brick
	 * @param command the command from <code>CommandEvent</code>
	 */
	public void sendCommand(byte command);
	
	/**
	 * Checks the bumper for an event.
	 * @return returns true if bumped
	 */
	public boolean getBumper();

	/**
	 * Terminates the Tower and Communitation, readies the programm to exit.
	 */
	public void close();

}